// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "CharacterController2D.hpp"

using namespace Dawn;

CharacterController2D::CharacterController2D(SceneItem *i) :
  SceneItemComponent(i)
{
}

void CharacterController2D::onStart() {
  useEvent([&](float_t delta){
    // Common variables
    auto myCollider = item->getComponent<Collider2D>();
    glm::vec2 currentPosition(
      this->transform->getLocalPosition().x,
      this->transform->getLocalPosition().z
    );

    // Friction
    velocity -= velocity * friction * delta;

    // Solid Collision Check
    glm::vec2 moveAmount;
    if(myCollider == nullptr) {
      moveAmount = velocity;
    } else {
      // Perform sweep
      auto allColliders = getScene()->findComponents<SolidController2D>();
      auto itColliders = allColliders.begin();

      struct CharacterController2DCollisionEventInfo info;
      bool_t result = false;

      // Check for collisions, definitely not working 100% yet 
      while(itColliders != allColliders.end()) {
        auto c = *itColliders;
        ++itColliders;
        if(c->item == this->item) continue;
        result = c->getCollidingResult(
          velocity,
          myCollider,
          info.normal,
          info.entryTime,
          info.exitTime,
          info.entryPoint,
          info.exitPoint
        ) && info.entryTime <= delta;

        if(result) {
          info.collider = c;
          break;
        }
      }
      
      if(result) {
        moveAmount = glm::vec2(0, 0);
        velocity = glm::vec2(0, 0);
        this->eventCollision.invoke(info);
      } else {
        moveAmount = velocity;
      }
    }

    if(
      mathAbs<float_t>(moveAmount.x) <= 0.001f &&
      mathAbs<float_t>(moveAmount.y) <= 0.001f
    ) return;

    transform->setLocalPosition(
      transform->getLocalPosition() + (glm::vec3(moveAmount.x, 0, moveAmount.y) * delta)
    );

    // Now perform trigger collision check
    auto allTriggers = getScene()->findComponents<TriggerController2D>();
    auto itTriggers = allTriggers.begin();
    while(itTriggers != allTriggers.end()) {
      auto c = *itTriggers;
      ++itTriggers;
      if(c->item == this->item) continue;
      if(c->getCollidingResult(myCollider)) {
        c->eventTriggerEnter.invoke(this);
      }
    }

  }, getScene()->eventSceneUpdate);
}