// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SolidController2D.hpp"

using namespace Dawn;

SolidController2D::SolidController2D(SceneItem *item) : SceneItemComponent(item) {
}

std::vector<SceneItemComponent*> SolidController2D::getDependencies() {
  return {
    (this->collider = item->getComponent<Collider2D>())
  };
}

bool_t SolidController2D::getCollidingResult(
  glm::vec2 movement,
  Collider2D *movingObject,
  glm::vec2 &normal,
  float_t &entryTime,
  float_t &exitTime,
  glm::vec2 &entryPoint,
  glm::vec2 &exitPoint
) {
  assertNotNull(this->collider);
  assertNotNull(movingObject);
  if(movement.x == 0 && movement.y == 0) return false;

  auto localPos = movingObject->transform->getWorldPosition();
  glm::vec2 myPos(localPos.x, localPos.z);

  // Check what the moving object is
  switch(movingObject->getColliderType()) {
    case COLLIDER2D_TYPE_BOX: {
      auto box1 = dynamic_cast<BoxCollider*>(movingObject);
      assertNotNull(box1);

      // Box VS (this)?
      switch(this->collider->getColliderType()) {
        case COLLIDER2D_TYPE_BOX: {
          auto box2 = dynamic_cast<BoxCollider*>(this->collider);
          assertNotNull(box2);
          auto localPos2 = box2->transform->getWorldPosition();
          glm::vec2 otherPos(localPos2.x, localPos2.z);

          return boxCheckCollision(
            myPos, box1->min, box1->max,
            otherPos, box2->min, box2->max,
            movement,
            normal, entryTime, exitTime, entryPoint, exitPoint
          );
        }
        
        default: {
          assertUnreachable();
        }
      }
      break;
    }

    default: {
      assertUnreachable();
    }
  }
  
  assertUnreachable();
  return false;
}