// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "EntityChargePlayer.hpp"

using namespace Dawn;

EntityChargePlayer::EntityChargePlayer(SceneItem* item) : SceneItemComponent(item) {
  
}

std::vector<SceneItemComponent*> EntityChargePlayer::getDependencies() {
  return {
    (characterController = item->getComponent<CharacterController2D>()),
    (this->entityMove = item->getComponent<EntityMove>())
  };
}

void EntityChargePlayer::onStart() {
  assertNotNull(characterController);
  assertNotNull(this->entityMove);

  useEvent([&](float_t delta) {
    // Get the direction to move in
    PlayerController * player = getScene()->findComponent<PlayerController>();
    if(player == nullptr) return;
    
    // Get the direction to move in
    glm::vec2 diff = glm::vec2(
      player->transform->getLocalPosition().x,
      player->transform->getLocalPosition().z
    ) - glm::vec2(
      transform->getLocalPosition().x,
      transform->getLocalPosition().z
    );

    if(glm::distance(glm::vec2(), diff) <= 2.0f) {
      this->entityMove->direction = glm::vec2(0, 0);
    } else {
      this->entityMove->direction = diff;
    }
  }, getScene()->eventSceneUpdate);
}