// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "EntityRandomWalk.hpp"

using namespace Dawn;

EntityRandomWalk::EntityRandomWalk(SceneItem* item) : SceneItemComponent(item) {
  
}

std::vector<SceneItemComponent*> EntityRandomWalk::getDependencies() {
  return {
    (characterController = item->getComponent<CharacterController2D>()),
    (this->entityMove = item->getComponent<EntityMove>())
  };
}

void EntityRandomWalk::onStart() {
  assertNotNull(characterController);

  useEvent([&](float_t delta) {
    // Do we need to pick a new destination
    if(timeLeftUntilNextDecision <= 0.0f) {
      // Pick a new destination to walk to.
      destination = randRange(-walkRange, walkRange);
      destination.x = mathAbs(destination.y) > mathAbs(destination.x) ? 0 : destination.x;
      destination.y = destination.x == 0 ? destination.y : 0;
      destination += glm::vec2(transform->getLocalPosition().x, transform->getLocalPosition().z);
      timeLeftUntilNextDecision = 4.0f;
    } else {
      timeLeftUntilNextDecision -= delta;
    }

    // Get the direction to move in
    glm::vec2 diff = destination - glm::vec2(
      transform->getLocalPosition().x,
      transform->getLocalPosition().z
    );
    if(glm::distance(glm::vec2(), diff) <= 1.0f) {
      this->entityMove->direction = glm::vec2(0, 0);
    } else {
      this->entityMove->direction = diff;
    }
  }, getScene()->eventSceneUpdate);
}